;----------------------------------------------------------------------
;
;   Filename:           pga.asm
;   Microcontroller:    PIC16F684
;   Assembled using:    MPLAB IDE v6.40
;
;   Author:             Steven Bible
;   Company:            Microchip Technology Inc.
;
;----------------------------------------------------------------------
;
;   Program Description:
;
;   < type program description here >
;
;----------------------------------------------------------------------
;
;   Revision History:
;
;   Ver. 0.0 - March 18, 2004: Initial write - sb
;   Ver. 0.1 - August 2004: Added PGA_Hysteresis - sb
;   Ver. 0.1a - September 2004: Modified PGA_Hysteresis for AN897 - sb
;
;----------------------------------------------------------------------
; Include Files and Project Files:
;

#include    "project.inc"           ; io port and processor defines
#include    "pga.inc"

    errorlevel  -302                ; suppress message 302 from list file

;----------------------------------------------------------------------
; Externally Defined Labels
;
; EXTERN declares symbol names that are used in this module and declared
; as global in another module.
;----------------------------------------------------------------------

    ; external variables

;   None

    ; external subroutines

    extern  SPI_PutByte             ; spi.asm
    extern  SPI_GetByte             ; spi.asm

;----------------------------------------------------------------------
; Global Defines
;----------------------------------------------------------------------

; see project.inc

;----------------------------------------------------------------------
; Local Defines
;----------------------------------------------------------------------

    ; none

;----------------------------------------------------------------------
; Variables
;----------------------------------------------------------------------

;--------------------
; Uninitialized Data Section
;
; The RES directive reserves space for data storage.

        udata

PGAdata     res 1                   ; PGA data variable (local)

PGAadc      res 2                   ; ADC reading variable (global)

;PGAtriplow  res 2                   ; low hysteresis trip value (global)
;PGAtriphigh res 2                   ; high hysteresis trip value (global)


;--------------------
; Shared Uninitialized Data Section
;
; Variables that are allocated in RAM that is shared across all RAM banks (unbanked RAM). 
; The RES directive reserves space for data storage.
;
; PIC16F684 has 16 shared RAM locations
;
        udata_shr

PGAgain     res 1                   ; PGA gain variable (global)


;--------------------
; Global Variables
;
; Variables defined in this module that are made available to other 
; modules. The <label> must be defined prior to using the GLOBAL
; directive in this module. 

    global  PGAgain
    global  PGAadc
;    global  PGAtriplow
;    global  PGAtriphigh


;----------------------------------------------------------------------
; Begin Executable Code Segment
;----------------------------------------------------------------------

;[<label>]   code    [<ROM_Address>]

        code

;----------------------------------------------------------------------
; Subroutine: Set PGA Gain
;   
; Description: Sets the gain setting in the PGA
;   
; Constants: 
;   PGA gain settings (defined in pga.inc):
;       PGAgain_1 = Gain +1
;       PGAgain_2 = Gain +2
;       PGAgain_4 = Gain +4
;       PGAgain_5 = Gain +5
;       PGAgain_8 = Gain +8
;       PGAgain_10 = Gain +10
;       PGAgain_16 = Gain +16
;       PGAgain_32 = Gain +32
;
; Global Variables: 
;   
; Initialization: 
;   1. W register contains the PGA gain setting to send
;   
; Output: 
;   None
;   
;----------------------------------------------------------------------

PGA_SetGain
        global  PGA_SetGain

        movwf   PGAdata             ; save PGA gain value

        bcf     PGA_CS              ; select PGA

        movlw   0x40                ; send PGA Gain Instruction
        call    SPI_PutByte

        movfw   PGAdata             ; send PGA gain value
        call    SPI_PutByte

        bsf     PGA_CS              ; deselect PGA

        return


;----------------------------------------------------------------------
; Subroutine: Set PGA Channel
;   
; Description: Sets the channel in the PGA
;   
; Constants: 
;   
; Global Variables: 
;   
; Initialization: 
;   1. W register contains the PGA channel setting to send
;   
; Output: 
;   None
;   
;----------------------------------------------------------------------

PGA_SetChan
        global  PGA_SetChan

        movwf   PGAdata             ; save PGA channel value

        bcf     PGA_CS              ; select PGA

        movlw   0x41                ; send PGA Channel Instruction
        call    SPI_PutByte

        movfw   PGAdata             ; send PGA channel value
        call    SPI_PutByte

        bsf     PGA_CS              ; deselect PGA

        return


;----------------------------------------------------------------------
; Subroutine: Shutdown PGA
;   
; Description: Shutdown the PGA
;   
; Constants: 
;   
; Global Variables: 
;   
; Initialization: 
;   
; Output: 
;   None
;   
;----------------------------------------------------------------------

PGA_Shutdown
        global  PGA_Shutdown

        bcf     PGA_CS              ; select PGA

        movlw   0x20                ; send PGA Shutdown Instruction
        call    SPI_PutByte

        movfw   0x00                ; send null value
        call    SPI_PutByte

        bsf     PGA_CS              ; deselect PGA

        return


;----------------------------------------------------------------------
; Subroutine: Set PGA Hysteresis
;   
; Description: Select PGA gain for hysteresis effect
;   
; Constants: 
;   PGA gain settings (defined in pga.inc):
;       PGAgain_1 = Gain +1
;       PGAgain_2 = Gain +2
;       PGAgain_4 = Gain +4
;       PGAgain_5 = Gain +5
;       PGAgain_8 = Gain +8
;       PGAgain_10 = Gain +10
;       PGAgain_16 = Gain +16
;       PGAgain_32 = Gain +32
;
; Global Variables: 
;   1. PGAgain
;   2. PGAadc+1:PGAadc = 16 bit variable containing ADC value
;
; Initialization: 
;   1. PGAgain = must be initialized prior to calling this routine
;   2. PGAtriplow+1:PGAtriplow = must be initialized to low trip value
;   3. PGAtriphigh+1:HIHGvalue = must be initialized to low trip value
;   
; Output: 
;   1. If PGAadc+1:PGAadc at trip point (HIGH or LOW) then PGA gain
;      is set to the next value. If PGA gain is changed, W is returned
;      with 1. Else, gain is not changed and W is returned with 0.
;   
;----------------------------------------------------------------------

PGA_Hysteresis
    global  PGA_Hysteresis

PGA1
        movfw   PGAgain             ; if (PGAgain = 1) {
        xorlw   PGA_GAIN_1
        btfss   STATUS, Z
         goto   PGA8

        movfw   PGAadc+1            ;   if (PGAadc < (113=0x0071))
        sublw   0x00
        btfss   STATUS, C
         goto   PGA_End_0           ;       else return 0
        btfss   STATUS, Z
         goto   PGA1_1
        movlw   0x71
        subwf   PGAadc, W
        btfsc   STATUS, C
         goto   PGA_End_0           ;       else return 0
PGA1_1
        movlw   PGA_GAIN_8          ;       PGAgain = 8;
        movwf   PGAgain
        call    PGA_SetGain
        goto    PGA_End_1           ;       return 1

PGA8
        movfw   PGAgain             ; } else if (PGAgain = 8) {
        xorlw   PGA_GAIN_8
        btfss   STATUS, Z
         goto   PGA32

        movfw   PGAadc+1            ;   if (PGAadc < (226=0x00E2))
        sublw   0x00
        btfss   STATUS, C
         goto   PGA8_2
        btfss   STATUS, Z
         goto   PGA8_1
        movlw   0xE2
        subwf   PGAadc, W
        btfsc   STATUS, C
         goto   PGA8_2
PGA8_1
        movlw   PGA_GAIN_32         ;       PGAgain = 32;
        movwf   PGAgain
        call    PGA_SetGain
        goto    PGA_End_1           ;       return 1
PGA8_2
        movlw   0x03                ;   else if (PGAadc > (960=0x03C0))
        subwf   PGAadc+1, W
        btfss   STATUS, C
         goto   PGA_End_0           ;       else return 0
        btfss   STATUS, Z
         goto   PGA8_3
        movfw   PGAadc
        sublw   0xC0
        btfsc   STATUS, C
         goto   PGA_End_0           ;       else return 0
PGA8_3
        movlw   PGA_GAIN_1          ;       PGAgain = 1;
        movwf   PGAgain
        call    PGA_SetGain
        goto    PGA_End_1           ;       return 1

PGA32
        movfw   PGAgain             ; } else if (PGAgain = 32) {
        xorlw   PGA_GAIN_32
        btfss   STATUS, Z
         goto   PGA_End_0

        movlw   0x03                ;   if (PGAadc > (960=0x03C0))
        subwf   PGAadc+1, W
        btfss   STATUS, C
         goto   PGA_End_0           ;       else return 0
        btfss   STATUS, Z
         goto   PGA32_3
        movfw   PGAadc
        sublw   0xC0
        btfsc   STATUS, C
         goto   PGA_End_0           ;       else return 0
PGA32_3
        movlw   PGA_GAIN_8          ;       PGAgain = 8;
        movwf   PGAgain
        call    PGA_SetGain
;        goto    PGA_End_1           ;       return 1

PGA_End_1
        retlw   1                   ; PGA gain was changed

PGA_End_0
        retlw   0                   ; PGA gain was _not_ changed


;----------------------------------------------------------------------
        end                         ; end of program directive
;----------------------------------------------------------------------
