;----------------------------------------------------------------------
; 
; Copyright 2005 Microchip Technology Inc.
; 
; Microchip Technology Inc. ("Microchip") licenses this software to
; you solely for use with Microchip products. The software is owned
; by Microchip and is protected under applicable copyright laws. All
; rights reserved.
; 
; SOFTWARE IS PROVIDED IN AN "AS IS." MICROCHIP EXPRESSLY DISCLAIMS ANY
; WARRANTY OF ANY KIND, WHETHER EXPRESS OR IMPLIED, INCLUDING BUT NOT
; LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
; PARTICULAR PURPOSE, OR NON-INFRINGEMENT. IN NO EVENT SHALL MICROCHIP
; BE LIABLE FOR ANY INCIDENTAL, SPECIAL, INDIRECT OR CONSEQUENTIAL
; DAMAGES, LOST PROFITS OR LOST DATA, HARM TO YOUR EQUIPMENT, COST OF
; PROCUREMENT OF SUBSTITUTE GOODS, TECHNOLOGY OR SERVICES, ANY CLAIMS
; BY THIRD PARTIES (INCLUDING BUT NOT LIMITED TO ANY DEFENSE THEREOF),
; ANY CLAIMS FOR INDEMNITY OR CONTRIBUTION, OR OTHER SIMILAR COSTS.
; 
;----------------------------------------------------------------------
;
;   Filename:           capacitance.asm
;   Microcontroller:    PIC16F690
;   Assembled using:    MPLAB IDE v7.20
;
;   Author:             Steven Bible
;   Company:            Microchip Technology Inc.
;
;----------------------------------------------------------------------
;
;   Program Description:
;
;   Reads capacitance using a dual slope integration technique. 
;
;----------------------------------------------------------------------
;
;   Revision History:
;
;   Ver. 0.1 - 2005-08-29: Initial write
;   Ver. 0.2 - 2005-11-22: Modified for relocateable module
;
;----------------------------------------------------------------------

    errorlevel  -302                ; suppress message 302 from list file

;----------------------------------------------------------------------
; Include Files
;----------------------------------------------------------------------

#include    "main.inc"              ; Project defines

;----------------------------------------------------------------------
; Global Defines
;----------------------------------------------------------------------

    ; Place the following #define directives in the main.inc file

;#define Vint        <port>, <port pin>

;----------------------------------------------------------------------
; Local Defines
;----------------------------------------------------------------------

; none

;----------------------------------------------------------------------
; Variables
;----------------------------------------------------------------------

;--------------------
; Uninitialized Data Section
;
; The RES directive reserves space for data storage.

        udata

COUNT           res 1               ; General Purpose counter variable

    ; K1 and K2 count variables

K1_H            res 1               ; 16-bit K1 count
K1_L            res 1

K2_U            res 1
K2_H            res 1               ; (Global) 16-bit K2 count
K2_L            res 1


;--------------------
; Shared Uninitialized Data Section
;
; Variables that are allocated in RAM that ais shared across all RAM banks (unbanked RAM). 
; The RES directive reserves space for data storage.

;        udata_shr

; none

;--------------------
; Global Variables
;
; Variables defined in this module that are made available to other 
; modules. The <label> must be defined prior to using the GLOBAL
; directive in this module. 

    global  K2_H
    global  K2_L


;----------------------------------------------------------------------
; Begin Executable Code Segment
;----------------------------------------------------------------------

;[<label>]  code    [<ROM_Address>]

        code

;----------------------------------------------------------------------
; Subroutine: Initialize Capacitive 
;   
; Description: Initializes Timer1 for use with this module.
;   
; Constants: 
;   
; Global Variables: 
;   
; Initialization: 
;   
; Output: 
;   
;----------------------------------------------------------------------

CAP_Init
    global  CAP_Init

;----------------------------------------
; TIMER1 Module with Gate Control (Section 6)
;
; The TIMER1 Control Register (T1CON) is used to enable/disable TIMER1
; and select various features of the TIMER1 module.

        banksel T1CON               ; Bank 0

        bcf     T1CON, TMR1CS       ; TIMER1 Clock Source Select: Internal Clock (Fosc/4)

        ; If TMR1CS=0 this bit is ignored
        bsf     T1CON, NOT_T1SYNC   ; TIMER1 External Clock Input Sync Control: do not syncronize external clock input

        ; T1OSCEN only if INTOSC without CLKOUT oscillator is active, else ignored
        bcf     T1CON, T1OSCEN      ; LP Oscillator Enable Control: LP oscillator off

        ; TIMER1 Input Prescale Select: 1:1
        bcf     T1CON, T1CKPS1      ; TIMER1 Input Clock Prescale Select bit 1
        bcf     T1CON, T1CKPS0      ; TIMER1 Input Clock Prescale Select bit 0

        ; TMR1GE only if TMR1ON = 1, else ignored
        bsf     T1CON, TMR1GE       ; TIMER1 Gate Enable: Timer1 is on when T1G_NOT = 0

        bcf     T1CON, T1GINV       ; TIMER1 gate is not inverted

        bcf     T1CON, TMR1ON       ; TIMER1: disabled


        return


;----------------------------------------------------------------------
; Subroutine: Read Capacitance
;   
; Description: Read capacitance using a dual slope integration technique.
;              Capacitance is reported as counts from Timer1.
;   
; Constants: 
;   
; Global Variables: K2_H, K2_L
;   
; Initialization: call CAP_Init prior to calling this routine.
;   
; Output: K2_H:K2_L is a global variable contains 16-bit capacitance
;         value.
;   
;----------------------------------------------------------------------

CAP_Read
    global  CAP_Read

    ; Step 1 - Initialize

        ; Configure Comparator C1
        ; ( settling time = 10 us max )
        banksel CM1CON0
        ;         /---------------- 7 - C1ON: C1 Comparator is enable
        ;         |/--------------- 6 - C1OUT
        ;         ||/-------------- 5 - C1OE: C1OUT is internal only
        ;         |||/------------- 4 - C1POL: C1OUT logic is inverted
        ;         ||||/------------ 3 - Unimplemented
        ;         |||||/----------- 2 - C1R: C1VP connects to C1Vref output
        ;         ||||||/---------- 1 - C1CH1: C1VN connects to RC1/C12IN-
        ;         |||||||/--------- 0 - C1CH0:
        ;         ||||||||
        ;         76543210
        movlw   b'10010101'
        movwf   CM1CON0

        ; Configure Comparator C2
        ; ( settling time = 10 us max )
        ;         /---------------- 7 - C2ON: C2 Comparator is enable
        ;         |/--------------- 6 - C2OUT
        ;         ||/-------------- 5 - C2OE: C2OUT is present on RC4/C2OUT
        ;         |||/------------- 4 - C2POL: C2OUT logic is inverted
        ;         ||||/------------ 3 - Unimplemented
        ;         |||||/----------- 2 - C2R: C2VP connects to C2Vref output
        ;         ||||||/---------- 1 - C2CH1: C2VN connects to RC1/C12IN-
        ;         |||||||/--------- 0 - C2CH0:
        ;         ||||||||
        ;         76543210
        movlw   b'10110101'
        movwf   CM2CON0

        ;         /---------------- 7 - MC1OUT
        ;         |/--------------- 6 - MC2OUT
        ;         ||/-------------- 5 - Unimplemented
        ;         |||/------------- 4 - Unimplemented
        ;         ||||/------------ 3 - Unimplemented
        ;         |||||/----------- 2 - Unimplemented
        ;         ||||||/---------- 1 - T1GSS: Timer1 gate source is RA4/T1G_NOT
        ;         |||||||/--------- 0 - C2SYNC: C2 output is asynchronous
        ;         ||||||||
        ;         76543210
        movlw   b'00000010'
        movwf   CM2CON1

        ; Configure Comparator Reference
        ; ( settling time = 10 us max )
        ;         /---------------- 7 - C1VREN: CVref on and routed to C1Vref
        ;         |/--------------- 6 - C2VREN: CVref on and routed to C2Vref
        ;         ||/-------------- 5 - VRR: Low range
        ;         |||/------------- 4 - VP6EN: disabled
        ;         ||||/------------ 3 - VR3: 0.125 * Vdd
        ;         |||||/----------- 2 - VR2:
        ;         ||||||/---------- 1 - VR1:
        ;         |||||||/--------- 0 - VR0:
        ;         ||||||||
        ;         76543210
        movlw   b'11100011'
        movwf   VRCON

        ; Configure SR Latch
        banksel SRCON
        ;         /---------------- 7 - SR1: SR latch enabled. C2OUT I/O pin is latch inverting output
        ;         |/--------------- 6 - SR0:
        ;         ||/-------------- 5 - C1SEN: C1 output has no effect on latch
        ;         |||/------------- 4 - C2REN: C2 output has no effect on latch
        ;         ||||/------------ 3 - PULSS: no pulse
        ;         |||||/----------- 2 - PULSR: pulse Reset input (Reset the latch)
        ;         ||||||/---------- 1 - Unimplemented
        ;         |||||||/--------- 0 - Unimplemented
        ;         ||||||||
        ;         76543210
        movlw   b'10000100'
        movwf   SRCON

        ; ramp down

        banksel PORTC
        bsf     Vint                ; Vint digital output (Vdd)

        ; make sure Vsen is less than 0.125 * Vdd

        banksel CM1CON0             ; compare CVref ( 0.215 * Vdd ) to Vsen
        btfsc   CM1CON0, C1OUT      ;   is CVref > Vsen?
         goto   $ - 1               ;       no, wait until CVref > Vsen

        banksel COUNT
        clrf    COUNT               ; wait 0.5 us * 3 * 256 = 384 us
        decfsz  COUNT, F
         goto   $ - 1

    ; Step 2 - begin ramp up, start K1 count at 0.125 * Vdd and
    ;          stop count at 0.500 * Vdd.

        banksel TMR1H
        clrf    TMR1H               ; clear Timer1
        clrf    TMR1L
        bsf     T1CON, TMR1ON       ; enable Timer1 (should not be counting because
                                    ;   SR latch reset Q_NOT = C2OUT = T1G_NOT = 1)
        ; Configure SR Latch
        banksel SRCON
        ;           /-------------- 5 - C1SEN: C1 output sets SR latch
        ;           |/------------- 4 - C2REN: C2 output has no effect on latch
        ;           ||
        ;           || /----------- 2 - PULSR: pulse Reset input (Reset the latch)
        ;           || |
        ;         76543210
        movlw   b'10100100'
        movwf   SRCON

        ; ramp up

        banksel PORTC
        bcf     Vint                ; Vint digital output (Ground)

        ; At this point Vsen is ramping up. 
        ; When Vsen crosses C1Vref ( 0.125 * Vdd ) then CVref < Vsen and
        ;   C1OUT_NOT = 1 and sets the SR latch.
        ; SR latch Q_NOT = C2OUT = T1G_NOT = 0 and enables Timer1 to count.

        ; detect when Vsen crosses C1Vref ( 0.125 * Vdd ) then reconfigure
        ;   the voltage reference to 0.500 * Vdd

        banksel CM1CON0             ; compare CVref ( 0.125 * Vdd ) to Vsen
        btfss   CM1CON0, C1OUT      ;   is CVref < Vsen?
         goto   $ - 1               ;       no, wait until CVref < Vsen

        ; change CVref to 0.500 * Vdd
        ; ( settling time = 10 us max )
        ;         /---------------- 7 - C1VREN: CVref on and routed to C1Vref
        ;         |/--------------- 6 - C2VREN: CVref on and routed to C2Vref
        ;         ||/-------------- 5 - VRR: Low range
        ;         |||/------------- 4 - VP6EN: disabled
        ;         ||||/------------ 3 - VR3: 0.500 * Vdd
        ;         |||||/----------- 2 - VR2:
        ;         ||||||/---------- 1 - VR1:
        ;         |||||||/--------- 0 - VR0:
        ;         ||||||||
        ;         76543210
        movlw   b'11101100'
        movwf   VRCON

        ; reconfigure SR Latch to reset when Vsen crosses the CVref voltage
        banksel SRCON
        ;           /-------------- 5 - C1SEN: C1 output has no effect on latch
        ;           |/------------- 4 - C2REN: C2 output resets SR latch
        ;           ||
        ;         76543210
        movlw   b'10010000'
        movwf   SRCON

        ; When CVref ( 0.500 * Vdd ) < Vsen then C2OUT_NOT = 1 and 
        ;   resets the SR latch.
        ; SR latch Q_NOT = C2OUT = T1G_NOT = 1 and disables Timer1.

        ; detect when CVref < Vsen and record the Timer1 value

        banksel CM1CON0             ; compare CVref ( 0.500 * Vdd ) to Vsen
        btfss   CM1CON0, C1OUT      ;   is CVref < Vsen?
         goto   $ - 1               ;       no, wait until CVref < Vsen

        banksel T1CON
        bcf     T1CON, TMR1ON       ; disable Timer1

        movfw   TMR1H               ; save K1 count
        movwf   K1_H
        movfw   TMR1L
        movwf   K1_L

        banksel COUNT
        clrf    COUNT               ; wait 0.5 us * 3 * 256 = 384 us
        decfsz  COUNT, F
         goto   $ - 1

    ; Step 3 - begin ramp down, start k2 count at 0.500 * Vdd and
    ;          stop count at 0.125 * Vdd

        banksel TMR1H
        clrf    TMR1H               ; clear Timer1
        clrf    TMR1L
        bsf     T1CON, TMR1ON       ; enable Timer1 (should not be counting because
                                    ;   SR latch reset: Q_NOT = C2OUT = T1G_NOT = 1)
        ; Configure Comparator C1
        ; ( settling time = 10 us max )
        banksel CM1CON0
        bcf     CM1CON0, C1POL      ; C1OUT logic is not inverted

        ; Configure Comparator C2
        ; ( settling time = 10 us max )
        bcf     CM2CON0, C2POL      ; C2OUT logic is not inverted

        ; Configure SR Latch
        banksel SRCON
        ;           /-------------- 5 - C1SEN: C1 output sets SR latch
        ;           |/------------- 4 - C2REN: C2 output has no effect on latch
        ;           ||
        ;           || /----------- 2 - PULSR: pulse Reset input (Reset the latch)
        ;           || |
        ;         76543210
        movlw   b'10100100'
        movwf   SRCON

        ; ramp down

        banksel PORTC
        bsf     Vint                ; Vint digital output (Vdd)

        ; At this point Vsen is ramping down.
        ; When Vsen crosses C1Vef ( 0.500 * Vdd ) then CVref > Vsen and
        ;   C1OUT = 1 and sets the SR latch.
        ; SR latch Q_NOT = C2OUT = T1G_NOT = 0 and enables Timer1 to count

        ; detect when Vsen crosses C1Vref ( 0.500 * Vdd ) then reconfigure
        ;   the voltage reference to 0.125 * Vdd

        banksel CM1CON0             ; compare CVref ( 0.125 * Vdd ) to Vsen
        btfss   CM1CON0, C1OUT      ;   is CVref > Vsen?
         goto   $ - 1               ;       no, wait until CVref > Vsen
        
        ; change CVref to 0.25 * Vdd
        ; ( settling time = 10 us max )
        ;         /---------------- 7 - C1VREN: CVref on and routed to C1Vref
        ;         |/--------------- 6 - C2VREN: CVref on and routed to C2Vref
        ;         ||/-------------- 5 - VRR: Low range
        ;         |||/------------- 4 - VP6EN: disabled
        ;         ||||/------------ 3 - VR3: 0.125 * Vdd
        ;         |||||/----------- 2 - VR2:
        ;         ||||||/---------- 1 - VR1:
        ;         |||||||/--------- 0 - VR0:
        ;         ||||||||
        ;         76543210
        movlw   b'11100011'
        movwf   VRCON

        ; reconfigure SR Latch to reset when Vsen crosses the CVref voltage
        banksel SRCON
        ;           /-------------- 5 - C1SEN: C1 output has no effect on latch
        ;           |/------------- 4 - C2REN: C2 output resets SR latch
        ;           ||
        ;         76543210
        movlw   b'10010000'
        movwf   SRCON

        ; When CVref ( 0.125 * Vdd ) > Vsen then C2OUT = 1 and 
        ;   resets the SR latch.
        ; SR latch Q_NOT = C2OUT = T1G_NOT = 1 and disables Timer1.

        ; detect when CVref > Vsen and record the Timer1 value

        banksel CM1CON0             ; compare CVref ( 0.125 * Vdd ) to Vsen
        btfss   CM1CON0, C1OUT      ;   is CVref > Vsen?
         goto   $ - 1               ;       no, wait until CVref > Vsen

        banksel T1CON
        bcf     T1CON, TMR1ON       ; disable Timer1

        clrf    K2_U
        movfw   TMR1H               ; save K2 count
        movwf   K2_H
        movfw   TMR1L
        movwf   K2_L


        banksel COUNT
        clrf    COUNT               ; wait 0.5 us * 3 * 256 = 384 us
        decfsz  COUNT, F
         goto   $ - 1

    ; Step 4 - Calculate capacitance

        movfw   K1_L                ; add K1 and K2, answer in K2
        addwf   K2_L, F

        btfsc   STATUS, C
         incf   K2_H, F

        movfw   K1_H
        addwf   K2_H, F

        btfsc   STATUS, C
         incf   K2_U, F


        bcf     STATUS, C           ; divide answer by 2 (result is average)
        rrf     K2_U, F
        rrf     K2_H, F
        rrf     K2_L, F


        return


;----------------------------------------------------------------------
        end                         ; end of program directive
;----------------------------------------------------------------------
